/*
 * Created on 11 jun 2009
 */

package jawnae.js;

import org.mozilla.javascript.Function;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;

public class JsFunction extends JsScope
{
   public JsFunction(String code)
   {
      this(code, null);
   }

   JsFunction(String code, JsScope scope)
   {
      this(code, scope, "<runtime>", 1);
   }

   public JsFunction(String code, JsScope scope, String sourceLocation, int lineOffset)
   {
      super(scope);

      // make it possible to write: "function sum(a, b) { return a + b}";
      // make it possible to write: "sum(a, b) { return a + b}";
      if (!code.trim().startsWith("function"))
      {
         code = "function " + code;
      }

      this.code = code;

      this.setName(this.findName());

      try
      {
         this.enterContext();

         this.rhinoFunction = this.rhinoContext.compileFunction(this.rhinoScope, code, sourceLocation, lineOffset, null);
      }
      finally
      {
         this.exitContext();
      }

      if (scope != null)
      {
         scope.putFunction(this);
      }
   }

   //

   private final String code;

   public String getCode()
   {
      return this.code;
   }

   //

   public JsScope getPrototype()
   {
      Scriptable proto = ScriptableObject.getObjectPrototype(this.rhinoFunction);
      return new JsScope(this.rhinoContext, proto);
   }

   //

   final Function rhinoFunction;

   public Object invoke(Object... args)
   {
      try
      {
         this.enterContext();

         return this.rhinoFunction.call(this.rhinoContext, this.rhinoScope, null, args);
      }
      finally
      {
         this.exitContext();
      }
   }

   //

   private String findName()
   {
      // so we are just going to read the sourcecode of this
      // function, so that we can hopefully grab the name
      // from it. once we found it, we are just going to bind
      // this function as a local variable on the current scope

      String code = this.getCode();
      int off = code.indexOf("function");
      if (off == -1)
         throw new IllegalStateException("failed to find function");
      off += ("function").length();

      int end = code.indexOf("(", off);
      if (end == -1)
         throw new IllegalStateException("failed to find function name");

      return code.substring(off, end).trim();
   }

   public String toString()
   {
      try
      {
         return "JsFunction[" + this.findName() + "]";
      }
      catch (Exception exc)
      {
         return "JsFunction[_anonymous_]";
      }
   }

   // convenience stuff

   /*
   
   @SuppressWarnings("all")
   public Object invoke(int... values)
   {
      Object[] args = new Object[values.length];
      for (int i = 0; i < args.length; i++)
         args[i] = Integer.valueOf(values[i]);
      return this.invoke(args);
   }

   @SuppressWarnings("all")
   public Object invoke(long... values)
   {
      Object[] args = new Object[values.length];
      for (int i = 0; i < args.length; i++)
         args[i] = Long.valueOf(values[i]);
      return this.invoke(args);
   }

   @SuppressWarnings("all")
   public Object invoke(float... values)
   {
      Object[] args = new Object[values.length];
      for (int i = 0; i < args.length; i++)
         args[i] = Float.valueOf(values[i]);
      return this.invoke(args);
   }

   @SuppressWarnings("all")
   public Object invoke(double... values)
   {
      Object[] args = new Object[values.length];
      for (int i = 0; i < args.length; i++)
         args[i] = Double.valueOf(values[i]);
      return this.invoke(args);
   }
   
   */
}